'<ADbasic Header, Headerversion 001.001>
' Foldings                       = 47,54,78,136,153,238,415,574,596
'<Header End>
'ADBASIC-SYSTEM-FILE--V1.00--0000--0000--INC--GOLD
'
'###############################################################
'#                 ADWINGOLDIITDrv.INC                         #
'#              INCLUDE-file for ADwin-GOLD-II-system          #
'###############################################################
'#                                                             #
'# Contents: Driver functions (running on the processor T11)   #
'#            for communicating with the TiCo processor        #
'#                                                             #
'#  18.11.2006  MF  creation                                   #
'#  05.07.2007  MF  Update                                     #
'#  17.07.2007  PR  Update (MSH-Addresses)                     #
'#  25.07.2007  MF SET /GET Data                               #
'#  16.08.2007  ur proc_no renamed to tico_no                  #
'#  22.08.2007  mf  P2_Set_TiCo_RingBuffer                     #
'#  06.09.2007  mf get_set Data                                #
'#  19.10.2007  mf COMM => Master                              #
'#  31.10.2007  MF HIDE                                        #
'#  06.12.2007  mf  TiCo_Start_Process                         #
'#  06.12.2007  mf  TiCo_Stop_Process                          #
'#  06.12.2007  mf  TiCo_Set_Processdelay                      #
'#  06.12.2007  mf  TiCo_Get_Processdelay                      #
'#  24.01.2008  Mf  Get/Set_TiCo_RingBuffer                    #
'#  25.01.2008  MF  get/set Memblock                           #
'#  31.01.2008  MF  datalocation removed                       #
'#  20.03.2008  gc  names of Sub/func parameter revised        #
'#  11.04.2008  mf  no_master removed                          #
'#  30.10.2008  mf  bugfix locate & lib                        # 
'#  20.03.2009  mf '''+                                        #
'#  13.05.2009  mf  bugfix Set_TiCo_RingBuffer                 #
'#  29.01.2010  mf  TiCo_Reset=>TiCo_Restart                   # 
'#  10.02.2010  mf  TiCo_Get_Array_Length                      #
'#  22.02.2010  mf  bugfix Set_Mem_Block                       #
'#  11.03.2010  mf  TDRV_Init Debug optimize                   #
'#  25.05.2010  mf  workload optimize                          #
'#  05.07.2010  mf  optimize Get_Par                           #
'#  09.07.2010  mf  optimize Set/GetData                       #
'#  27.07.2010  mf  optimize get/set Ringbuffer                #
'#  04.02.2016  mf  TiCo_Get_Process_Error                     #
'# Attention:                                                  #
'#   - Do not copy the first source line (containing           #
'#     "'ADBASIC-SYSTEM-FILE...") to user files !              #
'#     Otherwise the Syntax color may be not correct.          #
'###############################################################

'<REGION> Declaration
'Dim TDrv_last_time, TDrv_last_counter as Long ''<HIDE>


#Define TDRV_POFFSET 22 ''<HIDE>
'<ENDREGION>

'<REGION> Helper function

'************************************************
' Hardware
'*************************************************

'---------------------------------------------------------------
' If a low-priority process accesses a Pro-module it must not be
' interrupted by any other process !!!
'---------------------------------------------------------------

Sub Sub_Begin_Critical_Section() ''<HIDE>
#If PRIORITAET = LOW
  Begin_Critical_Section
#EndIf
EndSub

Sub Sub_End_Critical_Section() ''<HIDE>
#If PRIORITAET = LOW
  End_Critical_Section
#EndIf
EndSub
'<ENDREGION>

'<REGION> TiCo control
'********************************************************************
''' Stops TiCo.
'''******************************************************************
Sub TiCo_Stop()
  poke(1001E000h,0)
EndSub

'********************************************************************
''' Starts TiCo. Program must be loaded before.
'''******************************************************************
Sub TiCo_Start()
  poke(1001E000h, 1)
EndSub

'********************************************************************
''' Resets TiCo (stop/start). TiCo program must be loaded before.
'''******************************************************************
Sub TiCo_Reset() '<HIDE>
  poke(1001E000h,0)
  io_sleep(14)
  poke(1001E000h,1)
EndSub

'********************************************************************
''' Restarts TiCo (stop/start). TiCo program must be loaded before.
'''******************************************************************
Sub TiCo_Restart()
  poke(1001E000h,0)
  io_sleep(14)
  poke(1001E000h,1)
EndSub


'******************************************************************************
''' Reads TiCo Status.
'''+return 
'''+        0 = TiCo is stopped
'''+        1 = TiCo is running
'''****************************************************************************
Function Get_TiCo_Status() as Long
  dim a1_1, a1_2 as Long
  'count=peek(10010E00H)
  'if (count=0) then 'gold ii allways tico
  ' Get_TiCo_Status=-3
  'else
  a1_1   = peek(1001E220h)
  a1_2   = peek(1001E220h)
  if (a1_1=a1_2) then
    Get_TiCo_Status=0
  else
    Get_TiCo_Status=1
  endif
  'endif
EndFunction

'<ENDREGION>

'<REGION> Helper function uCore

'********************************************************************
' Helper functions only internal use
'*******************************************************************
Sub Reset_uCore()     ''<HIDE>
  poke(1001E000h, 0)
  io_sleep(100)
  poke(1001E000h, 1)
  io_sleep(100)
EndSub

Function Get_uCore_Register(adr) AS Long  ''<HIDE>
  Get_uCore_Register=peek(adr)
EndFunction
'<ENDREGION>

'<REGION> Par access
'******************************************************************************
''' Returns the value of the global variable PAR_x of a TiCo processor. 
''' tico_no: TiCo number (1..n)
''' par_no:  number of parameter to be read
'''****************************************************************************
Function Get_Par(tico_no, par_no) as Long
  Get_Par=peek(10080000H + par_no + TDRV_POFFSET)
EndFunction

'******************************************************************************
''' Transfers a number of global long variables, which is to be indicated, into an dest_array.
''' tico_no: TiCo number (1..n)
''' dest_array[]: destination array
''' dest_array_idx: start index in destination array
''' par_no: number of first parameter to be transferred
''' count: number of parameters to be transferred
'''****************************************************************************
Sub Get_Par_Block(tico_no, dest_array[], dest_array_idx, par_no, count)
  Dim TicoMem2[10000] as Integer Locate 10080000H ''<HIDE>

  Sub_Begin_Critical_Section
  memcpy(TicoMem2[1 + par_no + TDRV_POFFSET], dest_array[dest_array_idx], count)
  Sub_End_Critical_Section
EndSub

'******************************************************************************
' only for internal use !!!
' Transfers a number of global registers, which is to be indicated, into an array.
' tico_no: TiCo number (1..n)
' dest_array[]: destination array
' dest_array_idx: start index in destination array
' reg_addr: number of first register to be transferred
' count: number of registers to be transferred
'******************************************************************************
Sub Get_Mem_Block(tico_no, dest_array[], dest_array_idx, reg_addr, count) ''<HIDE>
  Dim TicoMem2[10000] as Integer Locate 10080000H ''<HIDE>
  
  Sub_Begin_Critical_Section
  memcpy(TicoMem2[1+reg_addr], dest_array[dest_array_idx], count)
  Sub_End_Critical_Section
EndSub

'******************************************************************************
''' Sets a global long variable to the specified value.
''' tico_no: TiCo number (1..n)
''' par_no: number of global Long variable
''' value: Value (-2^31.+2^31-1) of the global variable
'''****************************************************************************
Sub Set_Par(tico_no, par_no, value)
  poke(10080000H + par_no + TDRV_POFFSET ,value)
EndSub

'******************************************************************************
''' Transfers a number of array values into global long variables.
''' tico_no: TiCo number (1..n)
''' src_array[]: source array
''' src_array_idx: start index in source array
''' par_no: number of first value to be transferred
''' count: number of variables to be transferred
'''****************************************************************************
Sub Set_Par_Block(tico_no, src_array[], src_array_idx, par_no, count)
  Dim TicoMem2[10000] as Integer Locate 10080000H ''<HIDE>
  Sub_Begin_Critical_Section
  memcpy(src_array[src_array_idx], TicoMem2[1+par_no + TDRV_POFFSET], count)
  Sub_End_Critical_Section
EndSub

'******************************************************************************
' only for internal use !!!
' Transfers a number of array values into global registers.
' tico_no: TiCo number (1..n)
' src_array[]: source array
' src_array_idx: start index in source array
' reg_addr: number of first register to be transferred
' count: number of registers to be transferred
'******************************************************************************
Sub Set_Mem_Block(tico_no, src_array[], src_array_idx, reg_addr, count) ''<HIDE>
  Dim TicoMem2[10000] as Integer Locate 10080000H ''<HIDE>
  Sub_Begin_Critical_Section
  memcpy(src_array[src_array_idx], TicoMem2[1 + reg_addr], count)
  Sub_End_Critical_Section
EndSub
'<ENDREGION>

'<REGION> Process control
'******************************************************************************
' tico_no: TiCo number (1..n)
' process_no: number of process (1..4)
'******************************************************************************
Sub Set_Processdelay(tico_no, process_no, value) ''<HIDE>
  poke(10080000H + process_no + 151, value))
  poke(1001E24EH, 1)' necessary, if Eventsource is "Timer" or "Extern"
EndSub

'******************************************************************************
''' Sets the Processdelay (cycle time) of a TiCo process. 
'''+ before use, please first call TDRV_INIT
''' tdrv_datatable[]: array with 150 elements of type Long
''' process_no: number of process (1..4)
''' value: value to be set for Processdelay
'''****************************************************************************
Sub TiCo_Set_Processdelay(tdrv_datatable[], process_no, value)
  poke(10080000H + process_no + 151, value))
  If (tdrv_datatable[64+process_no] AND 1=0) then
    poke(1001E24EH, 1)' necessary, if Eventsource is "Timer" or "Extern"
  endif
EndSub

'******************************************************************************
' tico_no: TiCo number (1..n)
' process_no: number of process (1..4)
'******************************************************************************
Function Get_Processdelay(tico_no, process_no) as long ''<HIDE>
  Get_Processdelay=peek(10080000H + process_no + 151)
EndFunction

'******************************************************************************
''' Returns the Processdelay (cycle time) of a process on a TiCo processor. 
'''+ before use, please first call TDRV_INIT
''' tdrv_datatable[]: array with 150 elements of type Long  
''' process_no: number of process (1..4)
'''****************************************************************************
Function TiCo_Get_Processdelay(tdrv_datatable[], process_no) as Long
  'datatable later use for tico_nr
  TiCo_Get_Processdelay=peek(10080000H + process_no + 151)
EndFunction


'******************************************************************************
''' Returns the Debug Error Code of a process on a TiCo processor. 
'''+ before use, please first call TDRV_INIT
''' tdrv_datatable[]: array with 150 elements of type Long  
''' process_no: number of process (1..4)
'''****************************************************************************
Function TiCo_Get_Process_Error(tdrv_datatable[], process_no) as Long ''<HIDE>
  'datatable later use for tico_nr
  TiCo_Get_Process_Error=peek(10080000H + process_no + 171)
EndFunction



'******************************************************************************
''' Starts a process.
''' tico_no: TiCo number (1..n)
''' process_no: number of process (1..4)
'''****************************************************************************
Sub Start_Process_old(tico_no, process_no) ''<HIDE>
  poke(10080000H + process_no + 159,1)
  poke(1001E24EH, 1)' necessary, if Eventsource is "Timer" or "Extern"
EndSub

'******************************************************************************
''' Starts a process on a TiCo processor.
'''+ before use, please first call TDRV_INIT
''' tdrv_datatable[]: array with 150 elements of type Long  
''' process_no: number of process (1..4)
'''****************************************************************************
Sub TiCo_Start_Process(tdrv_datatable[], process_no)
  poke(10080000H + process_no + 159,1)
  If (tdrv_datatable[64+process_no] AND 1=0) then
    poke(1001E24EH, 1)' necessary, if Eventsource is "Timer" or "Extern"
  endif
EndSub

'******************************************************************************
' Stops a process.
' tico_no: TiCo number (1..n)
' process_no: number of process (1..4)
'******************************************************************************
Sub Stop_Process_old(tico_no, process_no) ''<HIDE>

  dim status as long
  Sub_Begin_Critical_Section
  status=peek(10080000H + process_no + 159)
  if (status=1) then poke(10080000H + process_no + 159,-1)
  Sub_End_Critical_Section

  poke(1001E24EH, 1)' necessary, if Eventsource is "Timer" or "Extern"
EndSub

'******************************************************************************
''' Stops a process on a TiCo processor.
'''+ before use, please first call TDRV_INIT
''' tdrv_datatable[]: array with 150 elements of type Long  (see comment at the top
''' process_no: number of process (1..4)
'''****************************************************************************
Sub TiCo_Stop_Process(tdrv_datatable[], process_no)
  
  dim status as long
  Sub_Begin_Critical_Section
  status=peek(10080000H + process_no + 159)
  if (status=1) then poke(10080000H + process_no + 159,-1)
  Sub_End_Critical_Section
  
  
  If (tdrv_datatable[64+process_no] AND 1=0) then
    poke(1001E24EH, 1)' necessary, if Eventsource is "Timer" or "Extern"
  endif
EndSub

'******************************************************************************
''' Returns the status of a TiCo process.
''' tico_no: TiCo number (1..n)
''' process_no: number of process (1..4)
'''****************************************************************************
Function Process_Status(tico_no, process_no) as long
  Process_Status=peek(10080000H + process_no + 159)
EndFunction

'******************************************************************************
''' Returns the processor workload.
'''+ before use, please first call TDRV_INIT
''' tdrv_datatable[]: array with 150 elements of type Long  
'''****************************************************************************
Function Workload(tdrv_datatable[]) as Float
  DIM time,counter,help_time,help_counter as Long
  
  dim proc_no as Long
  dim kor,i,d,s as long
  

  proc_no=tdrv_datatable[86]
  Sub_Begin_Critical_Section
  time    = peek(1001E220h)
  counter = peek(1001E240h)
  Sub_End_Critical_Section
  help_time    = time    - tdrv_datatable[87]
  help_counter = counter - tdrv_datatable[88]
  
  kor=0
  'if (tdrv_datatable[91]=1) then 'gold II always tico 1
  for i=1 to 4
    s=Process_Status(proc_no, i)
    if (s=1) then 'luft process i?
      
      'Bit 2-4:
      '000 Timer
      '001 Extern
      '010 None
      if (tdrv_datatable[64+i] AND 11100B=0) then 'nur bei timer process kor 
        d=help_time/Get_Processdelay(proc_no, i) 'as Long 
        kor=kor+d
      endif
    endif
  next
  help_counter=help_counter-kor
  'endif
  
  
  if (help_time = 0) then
    Workload = -1
  else
    Workload = 100 * ( 1 - help_counter / help_time)
  endif
  
  if (Workload>100) then workload=100
  tdrv_datatable[87] = time
  tdrv_datatable[88] = counter
EndFunction
'<ENDREGION>

'<REGION> Data
'******************************************************************************
''' Reads values from a global array of a TiCo processor and writes the values into a specified array.
'''+ before use, please first call TDrv_Init
''' tdrv_datatable[]: array with 150 elements of type Long  
''' src_array_no: source array number (DATA_x)
''' src_array_idx: number of first array element to be transferred
''' count: number of data elements to be transferred
''' dest_array[]: destination array
''' dest_array_idx: start index in the destination array
''' flowrate: 
'''+        1=low
'''+        2=medium
'''+        3=fast
'''*****************************************************************************
Sub GetData_Long(tdrv_datatable[], src_array_no, src_array_idx, count, dest_array[], dest_array_idx, flowrate)
  Dim TicoMem2[10000] as Integer Locate 10080000H ''<HIDE>
  dim datapointer as Long ' (absolute) pointer on the start of the DATA (on the element no. 0, which does not exist)
  ' dim datalocation as Long ' RAM  2 = intern 3 = extern
  dim datatype_reg as Long
  dim proc_no as Long
  DIM maxburst as Long
  DIM icount as Long
  DIM anzcount as Long
  DIM iindex as Long
  DIM istart as Long


#IF PRIORITAET = LOW
  maxburst=128
  if (flowrate=1) then maxburst=16
  if (flowrate=3) then maxburst=1024
#ELSE
  maxburst=32768
#ENDIF
  if (count>0) then
    icount=count
    iindex=dest_array_idx
    istart=src_array_idx

    proc_no = tdrv_datatable[86]
    ' PAR_92 is automatically incremented, when a TiCobasic process is loaded.
    ' If PAR_92 is changed, then the tdrv_datatable is invalid.
    ' => P2_TDRV_INIT has to be called again
    if (par_92<>tdrv_datatable[89]) then
      TDrv_Init(proc_no,tdrv_datatable)
    else

    endif
    datapointer= tdrv_datatable[1 + 4 * (src_array_no - 1)] ' (absolute) pointer on the start of the DATA
    ' datalocation=tdrv_datatable[2 + 4 * (src_array_no - 1)]
    datatype_reg=tdrv_datatable[3 + 4 * (src_array_no - 1)]
#IF DEBUG_MODE=ON THEN
    if (datatype_reg=-1) then
      PROZESS_FEHLER=210000  '  "TiCo DATA does not exist"
    else
      if ((datatype_reg AND 0FFFFH)>4) then PROZESS_FEHLER=210001   ' "TiCo DATA has wrong datatype"
      if (src_array_idx<1) then prozess_fehler=210003              ' "TiCo DATA-index < 1"
      if ((src_array_idx+count)>tdrv_datatable[4 + 4 * (src_array_no-1)]) then PROZESS_FEHLER=210002 ' "TiCo DATA-index too large"
    endif
#ENDIF
    if (datatype_reg <> -1) then ' check, if TiCo DATA exists
      do 
        anzcount=min_long(icount,maxburst)
        Sub_Begin_Critical_Section
        memcpy(TicoMem2[1 + datapointer + istart], dest_array[iindex], anzcount)
        Sub_End_Critical_Section
        icount=icount-anzcount
        iindex=iindex+anzcount
        istart=istart+anzcount
      until (icount=0)
    endif
  endif
EndSub

'******************************************************************************
''' Reads values from an array and writes them into a global array of a TiCo processor. 
'''+ before use, please first call TDrv_Init
''' tdrv_datatable[]: array with 150 elements of type Long  
''' dest_array_no: destination array number
''' dest_array_idx: number of first array element to be written
''' count: number of data elements to be transferred
''' src_array[]: source array
''' src_array_idx: start index in the source array
''' flowrate: 
'''+            1=low
'''+            2=medium
'''+            3=fast
'''******************************************************************************
Sub SetData_Long(tdrv_datatable[], dest_array_no, dest_array_idx, count, src_array[], src_array_idx, flowrate)
  Dim TicoMem2[10000] as Integer Locate 10080000H ''<HIDE>
  dim datapointer as Long ' (absolute) pointer on the start of the DATA (on the element no. 0, which does not exist)
  ' dim datalocation as Long 'RAM 2 = intern 3 = extern
  dim datatype_reg as Long
  dim proc_no as Long
  DIM maxburst as Long
  DIM icount, i as Long
  DIM anzcount as Long
  DIM iindex as Long
  DIM istart as Long
#IF PRIORITAET = LOW
  maxburst=128
  if (flowrate=1) then maxburst=16
  if (flowrate=3) then maxburst=1024
#ELSE
  maxburst=32768
#ENDIF
  if (count>0) then
    icount=count
    iindex=src_array_idx
    istart=dest_array_idx
    proc_no =    tdrv_datatable[86]
    ' PAR_92 is automatically incremented, when a TiCobasic process is loaded.
    ' If PAR_92 is changed, then the tdrv_datatable is invalid.
    ' => P2_TDRV_INIT has to be called again
    if (par_92<>tdrv_datatable[89]) then
      TDrv_Init(proc_no,tdrv_datatable)
    else

    endif
    datapointer= tdrv_datatable[1 + 4 * (dest_array_no - 1)] ' (absolute) pointer on the start of the DATA
    ' datalocation=tdrv_datatable[2 + 4 * (dest_array_no - 1)]
    datatype_reg=tdrv_datatable[3 + 4 * (dest_array_no - 1)]
#IF DEBUG_MODE=ON THEN
    if (datatype_reg = -1) then
      PROZESS_FEHLER=210020   '  "TiCo DATA does not exist"
    else
      if ((datatype_reg AND 0FFFFH) >4) then PROZESS_FEHLER=210021 ' "TiCo DATA has wrong datatype"
      if (dest_array_idx<1) then prozess_fehler=210023             ' "TiCo DATA-index < 1"
      if ((dest_array_idx+count)>tdrv_datatable[4 + 4 * (dest_array_no-1)]) then PROZESS_FEHLER=210022 ' "TiCo DATA-index too large"
    endif
#ENDIF
    if (datatype_reg <> -1) then ' check, if TiCo DATA exists
      do
        anzcount=min_long(icount,maxburst)
        Sub_Begin_Critical_Section
        memcpy(src_array[iindex],TicoMem2[1 + datapointer+istart],anzcount)
        Sub_End_Critical_Section
        icount=icount-anzcount
        iindex=iindex+anzcount
        istart=istart+anzcount
      until (icount=0)
    endif
  endif
EndSub

'******************************************************************************
''' Returns the length of TiCo Data.
'''+ -2 no data exists
''' tdrv_datatable[]: Array holding settings for data transfer, e. g. TiCo number.
''' datano: Number (1..16) of Data on the TiCo processor.
'''****************************************************************************
FUNCTION TiCo_Get_Array_Length(tdrv_datatable[], datano) as long
  if (par_92<>tdrv_datatable[89]) then TDRV_INIT(1,tdrv_datatable)
  TiCo_Get_Array_Length=tdrv_datatable[4 + 4 * ((datano) - 1)]-1  ' ringbuffer size (eventually unequal to declared size bec. 8- aligned)
ENDFUNCTION

'<ENDREGION>

'<REGION> General

'******************************************************************************
''' Gets data table from TiCo.
''' tico_no: TiCo number (1..n)
''' tdrv_datatable[]: array with 150 elements of type Long  
'''****************************************************************************
Sub TDrv_Init(tico_no, tdrv_datatable[])
  TDrv_Init_Int(tico_no, tdrv_datatable)
  
#IF DEBUG_MODE=ON THEN
  if (tdrv_datatable[90]<>16) then
    PROZESS_FEHLER=210500  '  "TiCo Datatable does not exist"
    end_exit
  endif
#ENDIF

EndSub


'<ENDREGION>

'<REGION> Ringbuffer
'******************************************************************************
''' Reads values from a ringbuffer of a TiCo processor and writes the values into an array of ADwin CPU.
'''+before use, please first call TDrv_Init
''' tdrv_datatable[]: array with 150 elements of type Long  
''' src_array_no: source array number
''' dest_array[]:destination array
''' dest_array_idx: startindex in the destination array
''' maxcount: Max. number (1.n) of transferred values.
''' flowrate: 
'''+         1=low
'''+         2=medium
'''+         3=fast
''' tico_par: Code number to transfer the read pointer to the TiCo processor.
'''+       1.80: Number (1.80) of global variable Par_n of the TiCo processor, into which the current read pointer value is written.
'''+       0: Read pointer value is not transferred
''' stuct: Code for measurement records:
'''+        0: Read arbitrary number of values.
'''+       >0: The number of transferred values must be a multiple of struct.
'''*****************************************************************************
Function Get_TiCo_RingBuffer(tdrv_datatable[], src_array_no, dest_array[], dest_array_idx, maxcount, flowrate, tico_par, struct) as Long
  dim datapointer as Long ' (absolute) pointer on the start of the DATA (on the element no. 0, which does not exist)
  ' dim datalocation as Long ' RAM  2 = intern 3 = extern
  dim datatype_reg as Long
  dim size as Long         ' ringbuffer size (eventually unequal to declared size bec. 8- aligned)

  dim tico_no as Long
  dim tindex as Long ' writepointer ( at first :  = absolute address in TiCo memory)
  '                                 ( later :  = rel. addr. in TiCo mem. ; range : 1 ... size )
  dim count as Long
  dim count1 as Long
  dim count2 as Long
  dim write_registerno as Long
  dim reg_no2 as Long         ' position of rel_read_pointer in tdrv_datatable
  dim rel_read_pointer as Long ' pointer relative to the start of the array :  1 ... n
  dim buffer_full as Long
  dim max as Long
#IF DEBUG_MODE=ON THEN
  dim read_registerno as Long
#ENDIF
  datapointer = tdrv_datatable[1 + 4 * (src_array_no - 1)] ' (absolute) pointer on the start of the DATA (on the element no. 0, which does not exist)
  ' datalocation= tdrv_datatable[2 + 4 * (src_array_no - 1)] ' RAM     2 = intern 3 = extern
  datatype_reg= tdrv_datatable[3 + 4 * (src_array_no - 1)]
  size =        tdrv_datatable[4 + 4 * (src_array_no - 1)]-1 ' ringbuffer size (eventually unequal to declared size bec. 8- aligned)
  tico_no =     tdrv_datatable[86]
  ' which TiCo Indexreg is used as a write pointer ?  Actually possible values : 1, 2
  write_registerno=shift_right(datatype_reg,16) AND 0FFH
  if (write_registerno<>0) then  ' Check : is this ringbuffer declared in TiCo for writing ? If not : do nothing
#IF DEBUG_MODE=ON THEN
    read_registerno = shift_right(datatype_reg,24)
    if (read_registerno<>0) then  ' Check : is this ringbuffer declared in TiCo also for reading ?
      ' Yes ! This means, that the TiCo AS WELL AS this Function wants to read
      ' from the ringbuffer. This will cause trouble  =>
      PROZESS_FEHLER=210100  '  "TiCo Ringbuffer is declared as READ and WRITE"
    endif
#ENDIF
    ' if (datalocation=2) then ' RAM  2 = intern
    reg_no2=74+write_registerno
    'else                      ' RAM  3 = extern
    ' reg_no2=78+write_registerno
    'endif
    rel_read_pointer=tdrv_datatable[reg_no2] ' pointer relative to the start of the array :  1 ... n
    count1=0
    count=0
    count2=0
    ' tindex = writepointer ( = absolute address in TiCo memory)
    'tindex = p2_get_uCore_Register(module,(3380E400h+32*(write_registerno-1))) ' get TiCos Index1 or Index2 Register
    tindex =peek(1001E400h+32*(write_registerno-1))' get TiCos Index1 or Index2 Register
    tindex = tindex - datapointer ' writepointer ( = relative address in TiCo memory ; range : 1 ... size )
    buffer_full = tindex - rel_read_pointer ' number of elements, which the ringbuffer contains
    if (buffer_full<0) then buffer_full=buffer_full+size ' turnaround - check
   
    max=min_long(buffer_full,maxcount)
   
    IF (struct>0) then max=(max/struct)*struct
    'if (tindex = rel_read_pointer) not necessary to handle this case, bec. then the ringbuffer is empty
    if (tindex < rel_read_pointer) then
      count=min_long((size - rel_read_pointer + 1),max)
      count1=count
      if (count > 0) then
        GetData_Long(tdrv_datatable,src_array_no,rel_read_pointer,count,dest_array,dest_array_idx,flowrate)
        rel_read_pointer = rel_read_pointer+count
        if (rel_read_pointer >  size) then rel_read_pointer=1   '
      endif
    endif
    if (tindex > rel_read_pointer) then   
      count=min_long((tindex - rel_read_pointer +1),(max-count1))
      count2=count
      if (count > 0) then
        GetData_Long(tdrv_datatable,src_array_no,rel_read_pointer,count,dest_array,dest_array_idx+count1,flowrate)
        rel_read_pointer = rel_read_pointer + count
      endif
    endif
    Get_TiCo_RingBuffer=count2+count1
    tdrv_datatable[reg_no2]=rel_read_pointer
    if ((tico_par>0) AND (tico_par<81)) then
      ' copy the (absolute) read_pointer into a TiCo parameter
      Set_Par(tico_no,tico_par,rel_read_pointer+datapointer)
    endif
  else
    Get_TiCo_RingBuffer=-1 ' error: ringbuffer was in TiCo not declared for writing
  endif
EndFunction

'******************************************************************************
''' Writes values from an ADwin CPU array into a ringbuffer on the TiCo processor.
'''+ before use, please first call TDRV_INIT
'''+ Success status of instruction:
'''+       -1: Error: The TiCo read ringbuffer is dimensioned wrongly.
'''+       0: Successful. The return value equals the number of transferred values.
''' tdrv_datatable[]: array with 150 elements of type Long  
''' dest_array_no: destination array number
''' src_array[]: source array
''' src_array_idx: startindex in the source array
''' maxcount:Max. number (1.n) of transferred values.
'''  flowrate: 
'''+          1=low
'''+          2=medium
'''+          3=fast
''' tico_par:  Code number to transfer the write pointer to the TiCo processor.
'''+          1.80: Number (1.80) of global variable Par_n of the TiCo processor, into which the current write pointer value is written.
'''+          0: Write pointer value is not transferred.
''' struct: Code for measurement records:
'''+          0: Read arbitrary number of values.
'''+         >0: The number of transferred values must be a multiple of struct.
'''****************************************************************************
Function Set_TiCo_RingBuffer(tdrv_datatable[], dest_array_no, src_array[], src_array_idx, maxcount, flowrate, tico_par, struct) as Long
  dim datapointer as Long ' (absolute) pointer on the start of the DATA (on the element no. 0, which does not exist)
  ' dim datalocation as Long ' RAM  2 = intern 3 = extern
  dim datatype_reg as Long
  dim size as Long         ' ringbuffer size (eventually unequal to declared size bec. 8- aligned)
  dim tico_no as Long
  dim tindex as Long ' readpointer ( at first :  = absolute address in TiCo memory)
  '                                 ( later :  = rel. addr. in TiCo mem. ; range : 1 ... size )
  dim count as Long
  dim count1 as Long
  dim count2 as Long
  dim read_registerno as Long
  dim reg_no2 as Long        ' position of rel_write_pointer in tdrv_datatable
  dim rel_write_pointer as Long  ' pointer relative to the start of the array :  1 ... n
  dim buffer_full as Long
  dim max as Long
#IF DEBUG_MODE=ON THEN
  dim write_registerno as Long
#ENDIF
  datapointer = tdrv_datatable[1 + 4 * (dest_array_no - 1)] ' (absolute) pointer on the start of the DATA (on the element no. 0, which does not exist)
  'datalocation= tdrv_datatable[2 + 4 * (dest_array_no - 1)] ' RAM     2 = intern 3 = extern
  datatype_reg= tdrv_datatable[3 + 4 * (dest_array_no - 1)]
  size =        tdrv_datatable[4 + 4 * (dest_array_no - 1)]-1       ' ringbuffer size (eventually unequal to declared size bec. 8- aligned)
  tico_no =     tdrv_datatable[86]
  ' which TiCo Indexreg is used as a read pointer ?  Actually possible values : 1, 2
  read_registerno=shift_right(datatype_reg,24)
  if (read_registerno<>0) then  ' Check : is this ringbuffer declared in TiCo for reading ? If not : do nothing
#IF DEBUG_MODE=ON THEN
    write_registerno = shift_right(datatype_reg,16) AND 0FFH
    if (write_registerno<>0) then  ' Check : is this ringbuffer declared in TiCo also for writing ?
      ' Yes ! This means, that the TiCo AS WELL AS this Function wants to write
      ' to the ringbuffer. This will cause trouble  =>
      PROZESS_FEHLER=210120  '  "TiCo Ringbuffer is declared as READ and WRITE"
    endif
#ENDIF
    'if (datalocation=2) then ' RAM  2 = intern 3 = extern
    reg_no2=74+read_registerno
    'else
    ' reg_no2=78+read_registerno
    'endif
    rel_write_pointer=tdrv_datatable[reg_no2] ' pointer relative to the start of the array :  1 ... n
    count=0
    count1=0
    count2=0
    ' tindex = readpointer ( = absolute address in TiCo memory)
    'tindex = p2_get_uCore_Register(module,(3380E400h+32*(read_registerno-1))) ' get TiCos Index1 or Index2 Register
    tindex =peek(1001E400h+32*(read_registerno-1))' get TiCos Index1 or Index2 Register
    tindex = tindex - datapointer '  readpointer ( = relative address in TiCo memory ; range : 1 ... size )
    buffer_full = rel_write_pointer - tindex+1 ' number of elements, which the ringbuffer contains
    ' if (buffer_full<0) then buffer_full=buffer_full+size ' turnaround - check
    if (buffer_full<1) then buffer_full=buffer_full+size ' turnaround - check
   
    max=min_long((size-buffer_full),maxcount)' max. count of the free elements in the ringbuffer
   
    IF (struct>0) then max=(max/struct)*struct
    if (tindex <= rel_write_pointer) then 
      count=min_long((size - rel_write_pointer + 1),max)
      count1=count
      if (count > 0) then
        SetData_Long(tdrv_datatable,dest_array_no,rel_write_pointer,count,src_array,src_array_idx,flowrate)
        rel_write_pointer = rel_write_pointer+count
        if (rel_write_pointer>size) then rel_write_pointer=1
      endif
    endif
    if (tindex >= rel_write_pointer) then
      count=min_long((tindex - rel_write_pointer +1),(max-count1))
      count2=count
      if (count > 0) then
        SetData_Long(tdrv_datatable,dest_array_no,rel_write_pointer,count,src_array,src_array_idx+count1,flowrate)
        rel_write_pointer = rel_write_pointer + count
      endif
    endif
    Set_TiCo_RingBuffer=count2+count1
    tdrv_datatable[reg_no2]=rel_write_pointer
    if ((tico_par>0) AND (tico_par<81)) then
      ' copy the (absolute) write_pointer into a TiCo parameter
      Set_Par(tico_no,tico_par,rel_write_pointer+datapointer)
    endif
  else
    Set_TiCo_RingBuffer=-1 ' error : ringbuffer was in TiCo not declared for reading
  endif
EndFunction

'******************************************************************************
''' Returns the number of used elements in a read ringbuffer on a TiCo processor.
'''+ before use, please first call TDRV_INIT
''' tdrv_datatable[]: array with 150 elements of type Long (see comment at the top)
''' data_no: number of TiCo ringbuffer DATA
'''****************************************************************************
Function RingBuffer_Full(tdrv_datatable[], data_no) as Long
  dim datapointer as Long ' (absolute) pointer on the start of the DATA (on the element no. 0, which does not exist)
  ' dim datalocation as Long ' RAM  2 = intern 3 = extern
  dim datatype_reg as Long
  'dim size as Long
  dim mode as Long
  dim read_registerno as Long
  dim write_registerno as Long
  dim reg_no3 as Long
  dim write_pointer as Long
  dim read_pointer as Long
  datapointer = tdrv_datatable[1 + 4 * (data_no - 1)] ' (absolute) pointer on the start of the DATA
  'datalocation= tdrv_datatable[2 + 4 * (data_no - 1)] ' RAM  2 = intern 3 = extern
  datatype_reg= tdrv_datatable[3 + 4 * (data_no - 1)]
  'size =        tdrv_datatable[4 + 4 * (data_no - 1)]-1
  ' which TiCo Indexreg is used as read / write pointer ?  Actually possible values : 1, 2
  read_registerno=shift_right(datatype_reg,24) 'read
  write_registerno=shift_right(datatype_reg,16) AND 0FFH 'write
  mode=0
  '] =get_uCore_Register(1001E400h) 'Index1
  'Data_201[8] =get_uCore_Register(1001E420h) 'Index2
  if (read_registerno>0) then '
    mode=1        ' TiCo reads this ringbuffer
    read_pointer =peek(1001E400h+32*(read_registerno-1))' get TiCos Index1 or Index2 Register
    'read_pointer  = p2_get_uCore_Register(module,(3380E400h+32*(read_registerno-1)))  ' get TiCos Index1 or Index2 Register
  endif
  if (write_registerno>0) then '
    mode=mode+2  ' TiCo writes to this ringbuffer
    'write_pointer = p2_get_uCore_Register(module,(3380E400h+32*(write_registerno-1))) ' get TiCos Index1 or Index2 Register
    write_pointer =peek(1001E400h+32*(write_registerno-1))' get TiCos Index1 or Index2 Register
  endif
  reg_no3 = 74 ' read_ or write_registerno will be added later
  if (mode=0) then    ' type : no buffer at all
    RingBuffer_Full=-1  ' error
  else
    if (mode=1) then ' TiCo does ONLY read this ringbuffer
      read_pointer = read_pointer - datapointer ' make the absolute pointer to a relative pointer
      write_pointer=tdrv_datatable[reg_no3 + read_registerno] ' get (relative) pointer
    else
      if (mode=2) then ' TiCo does ONLY write to this ringbuffer
        write_pointer = write_pointer - datapointer ' make the absolute pointer to a relative pointer
        read_pointer=tdrv_datatable[reg_no3 + write_registerno] ' get (relative) pointer
        ' else
        '   (mode=3) nothing more to do for mode 3
      endif
    endif
    ' in mode 3 (read AND write) : substract absolute pointers
    ' in mode 1 and 2            : substract relative pointers
    RingBuffer_Full = write_pointer - read_pointer
    if (RingBuffer_Full < 0) then
      RingBuffer_Full=RingBuffer_Full + tdrv_datatable[4 + 4 * (data_no - 1)]-1 '  size = tdrv_datatable[4 + 4 * (data_no - 1)]-1
    endif
  endif
EndFunction

'******************************************************************************
''' Returns the number of free elements in a write ringbuffer on a TiCo processor.
'''+ before use, please first call TDRV_INIT
''' tdrv_datatable[]: array with 150 elements of type Long (see comment at the top)
''' data_no: number of TiCo ringbuffer DATA
'''****************************************************************************
Function RingBuffer_Empty(tdrv_datatable[], data_no) as Long
  'dim size as Long
  dim full as Long
  'size = tdrv_datatable[4 + 4 * (data_no - 1)]-1
  full=RingBuffer_Full(tdrv_datatable,data_no)
  if (full=-1) then
    RingBuffer_Empty=-1
  else
    RingBuffer_Empty=tdrv_datatable[4 + 4 * (data_no - 1)]-2 - full
  endif
EndFunction
'<ENDREGION>












