'<TiCoBasic Header, Headerversion 001.001>
' Foldings                       = 40,151,220
'<Header End>
'##############################################################################
'#                        CNT_TiCo.inc                                        #
'#              INCLUDE-file for PRO II CNT with Tico                         #
'##############################################################################
'#                                                                            #
'#                                                                            #
'#  04.12.2007  mf   creation                                                 #
'#  04.12.2007  mf   ssi                                                      #
'#  18.12.2007  mf   cnt...                                                   #
'#  01.02.2008  mf  #if adwin_system                                          #
'#  05.02.2008  MF   new fast adr                                             #
'#  11.02.2008  MF                                                            #
'#  19.02.2008  MF  CNT_Sync_Latch                                            #
'#  25.03.2009  MF                                                            #
'#  14.05.2009  MF  optimize CNT_Get_PW_HL                                    #
'#  30.01.2014  ap  Doc update                                                #
'#  04.11.2016  mf  Calc_Processdelay                                         #
'#  25.01.2021  mf  Calc_TicksToNs                                            #
'#                                                                            #
'# Attention:                                                                 #
'#   - Do not copy the first source line (containing                          #
'#     "'ADBASIC-SYSTEM-FILE...") to user files !                             #
'#     Otherwise the Syntax color may be not correct.                         #
'#   - This includefile requires ADbasic 4.20.04 or higher!                   #
'#                                                                            #
'##############################################################################

#if adwin_system=ADwin_ProII
#else
#error("This include file can only be used with the ADwin ProII")
#endif

'<REGION> Declaration

#If MAKE_LIB = OFF Then
dim PWM_help_mode as long  ''<HIDE>
#endif
'<ENDREGION>

'<REGION> System


'***************************************************************
''' Converts a process frequency [Hz] into processor ticks (processdelay or cycle time).
''' frequency:  frequency[Hz]
'''*************************************************************
Function Calc_Processdelay(frequency) as long
  
#if prozessor=tico1
  calc_processdelay=50000000/(frequency)
#else
  calc_processdelay=100000000/(frequency)
#endif
  
endfunction



'***************************************************************
''' Converts timer ticks into ns
''' ticks: counter value ticks into ns
'''*************************************************************
Function Calc_TicksToNs(ticks) As long
  Calc_TicksToNs=(ticks)*20
endfunction



'---------------------------------------------------------------
' If a low-priority process accesses must not be
' interrupted by any other process !!!
'---------------------------------------------------------------

Sub Sub_Begin_Critical_Section() ''<HIDE>
#If PRIORITAET = LOW
  Begin_Critical_Section
#EndIf
EndSub

Sub Sub_End_Critical_Section() ''<HIDE>
#If PRIORITAET = LOW
  End_Critical_Section
#EndIf
EndSub

'******************************************************************************
'''Switches the LED on or off.
'''value: 
'''+       0 off 
'''+       1 on
'''****************************************************************************
SUB Set_LED(value)
  out (80804000H, value)
ENDSUB

'******************************************************************************
'''Checks the LED.
'''+Return: 
'''+         0 off 
'''+         1 on
'''****************************************************************************
FUNCTION Check_LED() AS LONG
  Check_LED=in(80804000h)
ENDFUNCTION

'*******************************************************************************
'''Activates or deactivates the external Event input.
'''value: 
'''+       0 deactivate 
'''+       1 activate
'''*****************************************************************************
SUB Event_Enable(value)
  DIM old AS Long

  old=in(80804800h)
  old =(old AND 0FFFFFFFEH) or value
  out (80804800h, old)
ENDSUB

'*******************************************************************************
'''Configures the external Event input.
'''min_hold: 0/1  0 = 15 ns (default) 1 = 50 ns
'''edge: 
'''+         1 rising edge (default)
'''+         2 falling edge
'''+         3 rising & falling edge
'''prescale: 1 to 255
'''*****************************************************************************
SUB Event_Config(min_hold,edge,prescale)
  DIM old AS Long
  DIM neu AS Long
  DIM value AS Long

  old=in(80804800h)

  neu=shift_left((min_hold AND 1),4) or shift_left((edge AND 3),2)
  old =(old AND 0FFFFFFE3H) or neu
  out (80804800h, old)
  value=prescale AND 0FFH
  out (80804810h,value)
ENDSUB

'*******************************************************************************
'''Creates an event for the master cpu.
'''*****************************************************************************
SUB Trigger_Event()
  out (108041E0h, 1)
ENDSUB
'<ENDREGION>

'<REGION> SSI

'**************************************************************
'''Reads SSI encoder value 32 Bit.
'''channel: number of the ssi decoder (1..2)
'''************************************************************
FUNCTION SSI_Read(dcdr_no) AS long
  SSI_Read=in(000010C4H+dcdr_no)
ENDFUNCTION

'**************************************************************
'''Sets the number of databits for the SSI Interface.
'''dcdr_no: number of the ssi decoder (1..2)
'''bit_no: number of bits
'''************************************************************
SUB SSI_Set_Bits(dcdr_no, bit_no)
  out(000010C2h+dcdr_no,bit_no)
ENDSUB

'**************************************************************
'''Sets the frequency of the SSI clock.
'''prescale: frequency = 25 MHZ / prescale
'''************************************************************
SUB SSI_Set_Clock(prescale)
  out(000010C2h,prescale)
ENDSUB

'**************************************************************
'''Starts the SSI interface.
'''pattern: Bit combination each bit corresponds to one counter
'''************************************************************
SUB SSI_Start(pattern)
  out(000010C0H,pattern)
ENDSUB

'*************************************************************
'''Sets the SSI mode.
'''pattern: Bit combination each bit corresponds to one counter
'''+ 0 = single start mode
'''+ 1 = continuous start mode
'''***********************************************************
sub SSI_Mode(pattern)
  out(000010C1H,pattern)
endsub

'*************************************************************
'''Reads the SSI status.
'''dcdr_no: number of the SSI decoder
'''return value: 
'''+              0 = SSI value compleated
'''+              1 = SSI transfer running
'''***********************************************************
FUNCTION SSI_Status(dcdr_no) AS long
  dim value2 as long
  value2=in(000010C0H)
  SSI_Status=shift_right(value2,dcdr_no-1) AND 01H 'shift_right(value2,((dcdr_no)-1)) AND 01H
ENDFUNCTION

'*************************************************************
'''Sets the delay in continuous mode.
'''dcdr_no: number of the SSI decoder
'''delay: 16 Bit : / 20 ns  [default 25 s /1250]
'''***********************************************************
SUB SSI_Set_Delay(dcdr_no, delay)
  out(000010C9h+dcdr_no,delay)
ENDSUB

'<ENDREGION>

'<REGION> Counter

'***************************************************************
'''Sets mode for counter.
'''counter_no: number of the counter (1..4)
'''mode:
'''+   Bit 0 CNT MODE
'''+           0->clock/direction
'''+           1->AB
'''+   Bit 1 CLEAR MODE
'''+           0->normal mode
'''+           1->clear IF A=1 B=1 C=1
'''+   Bit 2 INV CLK
'''+           0->not inverse
'''+           1->inverse
'''+   Bit 3 INV DIR
'''+           0->not inverse
'''+           1->inverse
'''+   Bit 4 CLEAR or LATCH
'''+           0->C use for clear
'''+           1->C use for latch
'''+   Bit 5 Ext Clear/Latch Enable
'''+           0->disable
'''+           1->enable
'''+   Bit 6 PWM Reference edge
'''+           0->positive
'''+           1->negative
'''+   Bit 7, 8 PWM INPUT_SELECT
'''+           00->A
'''+           01->B
'''+           10->C
'''*************************************************************
SUB CNT_Mode(counter_no, mode)
  
#If MAKE_LIB = ON Then
  #error("CNT_Mode does NOT work in Libraries!")
#else
  
  DIM modei as Long
  'POKE_FAST_BUS((3200109Fh+(counter_no) OR shift_left(module,16)),mode)
  OUT((0000109Fh+(counter_no)),mode)
  modei=shift_right(mode,6) AND 1
  PWM_help_mode=((NOT(shift_left(1,counter_no-1))) AND PWM_help_mode) OR shift_left(modei,counter_no-1)

#endif

ENDSUB

'***************************************************************
'''Activates or deactivates counter.
'''pattern: Bit 0-3 VR counter Enable
'''*************************************************************
SUB CNT_Enable(pattern)
  out(000010B5h,pattern)
ENDSUB

'***************************************************************
'''Activates or deactivates PW counter.
'''pattern: Bit 0-3 VR PW Counter Enable
'''*************************************************************
SUB CNT_PW_Enable(pattern)
  out((000010B6h),pattern)
ENDSUB

'***************************************************************
'''Latches the counter value to register.
'''pattern: each bit corresponds to a counter.
'''*************************************************************
SUB CNT_Latch(pattern)
  out((000010B0h),pattern)
  nops(4)
ENDSUB

'***************************************************************
'''Reads the latch register.
'''counter_no: number of the counter (1..4)
'''*************************************************************
FUNCTION CNT_Read_Latch(counter_no) as Long
  CNT_Read_Latch=in((00000FF6h+(counter_no)*16))
ENDFUNCTION

'**************************************************************
'''Latches counter to register and reads counter value.
'''counter_no: number of the counter (1..4)
'''************************************************************
FUNCTION CNT_Read(counter_no) as Long
  out((000010B0h),shift_left(1,counter_no-1))
  nops(4)
  CNT_Read=in((00000FF6h+(counter_no)*16))
ENDFUNCTION

'***************************************************************
'''Resets counter.
'''pattern: each bit corresponds to a counter
'''*************************************************************
SUB CNT_Clear(pattern)
  out(000010B1h,pattern)
ENDSUB

'**************************************************************
'''Reads the status of a counter input.
'''+Return:
'''+          Bit 0= A/CLK Input
'''+          Bit 1= B Input
'''+          Bit 2= C/L Input
'''+          Bit 3=Cable error
'''+          Bit 4=Phase error
''' counter_no: number of the counter (1..4)
'''************************************************************
FUNCTION CNT_Get_Status(counter_no) as Long
  CNT_Get_Status=in(00000FFFh+(counter_no)*16)
ENDFUNCTION

'FUNCTION for spezial user
'**************************************************************
'''+Returns the content of a counter register.
'''counter_no: number of the counter (1..4)
'''regnr:
'''+    0 pos_pwm_latch 1
'''+    1 pos_pwm_latch 2
'''+    2 pos_pwm_latch 3
'''+    3 neg_pwm_latch 1
'''+    4 neg_pwm_latch 2
'''+    5 neg_pwm_latch 3
'''+    6 vr latch
'''+    7 pwm latch
'''+    8 pos_pwm_latch 1 shadow
'''+    9 pos_pwm_latch 2 shadow
'''+   10 pos_pwm_latch 3 shadow
'''+   11 neg_pwm_latch 1 shadow
'''+   12 neg_pwm_latch 2 shadow
'''+   13 neg_pwm_latch 3 shadow
'''+   14 vr_latch shadow
'''+   15 status
'''***************************************************************
FUNCTION CNT_Read_Int_Register(counter_no, regnr) as Long
  CNT_Read_INT_Register=in(00000FF0h+regnr+(counter_no)*16)
ENDFUNCTION

'***************************************************************
'''Copies pw-latches and vr-latches to shadow register.
'''pattern: each bit corresponds to a counter
'''*************************************************************
SUB CNT_PW_Latch(pattern)
  out(000010B2h,pattern)
  nops(4)
ENDSUB

'***************************************************************
'''Latches vr and pw counter to shadow register.
'''pattern: each bit corresponds to a counter
'''*************************************************************
SUB CNT_Sync_Latch(pattern)
  out(000010CFh,pattern)
  nops(4)
ENDSUB

'**************************************************************
'Reads pw frequency and dutycycle use cnt_pw_latch before.
'by val counter_no
'by ref frequency as long
'by ref dutycycle as long
'**************************************************************
SUB CNT_Get_PW(counter_no, frequency, dutycycle) ''<HIDE> use from TiCo CNT_Get_PW_HL
#If MAKE_LIB = ON Then
  #error("CNT_Get_PW does NOT work in Libraries!")
#else
  DIM period as Long
  DIM puls as Long
  DIM ref_edge as Long

  DIM AR1,AR2,AR3,AR4,AR5 as long

  AR2=in(00000FF9h+1+(counter_no)*16)
  'AR3=in(00000FF9h+2+(counter_no)*16)

  AR5=in(00000FF9h+4+(counter_no)*16)

  ref_edge=shift_right(PWM_help_mode,counter_no-1) AND 1

  IF (ref_edge=1) then 'negative referenz edge
    AR4=in(00000FF9h+3+(counter_no)*16)
    period=AR4-AR5 'neg2-neg3
    puls=AR4-AR2 'neg2-pos3
  ELSE 'positive referenz edge
    AR1=in(00000FF9h+0+(counter_no)*16)

    period=AR1-AR2 'pos2-pos3
    puls=AR5-AR2 'neg3-pos3
  ENDIF

  if (period<>0) then
    frequency=100000000/period 'Frequenz [Hz] '100 MHz
    dutycycle=puls*100/(period) 'PWM Verhltnis
  else
    frequency=0
    dutycycle=0
  endif
#endif
ENDSUB

'**************************************************************
'''Reads registers and calculates pw hightime and lowtime.
'''+Remark: use P2_cnt_pw_latch before (to get actual values)
'''counter_no: number of PW counter (1..4)
'''hightime: Pulse duration in units of 10 ns: PWM high level time by_ref
'''lowtime: Pulse period in units of 10 ns: PWM low level time by_ref
'''*************************************************************
SUB CNT_Get_PW_HL(counter_no, hightime, lowtime)
#If MAKE_LIB = ON Then
  #error("CNT_Get_PW_HL does NOT work in Libraries!")
#else
  DIM ref_edge as Long
  DIM AR1,AR2,AR3,AR4,AR5 as long

  
  AR2=in(00000FF9h+1+(counter_no)*16)
  ' AR3=in(00000FF9h+2+(counter_no)*16)

  AR5=in(00000FF9h+4+(counter_no)*16)


  ref_edge=shift_right(PWM_help_mode,counter_no-1) AND 1

  IF (ref_edge=1) then 'negative referenz edge
    '  period=neg2-neg3
    AR4=in(00000FF9h+3+(counter_no)*16)
    hightime=AR4-AR2 'neg2-pos3
    lowtime=AR2-AR5 'pos3-neg3
  ELSE 'positive referenz edge
    ' period=pos2-pos3
    AR1=in(00000FF9h+0+(counter_no)*16)
    hightime=AR5-AR2'neg3-pos3
    lowtime=AR1-AR5 'pos2-neg3
  ENDIF

#endif
ENDSUB

'<ENDREGION>


