'<TiCoBasic Header, Headerversion 001.001>
' Foldings                       = 24,105,139,183,206,330
'<Header End>
'ADBASIC-SYSTEM-FILE--V1.00--0000--0000--INC--PRO
'<REGION> About
'##############################################################################
'#                        DIO_Base_Tico.inc                                   #
'#              INCLUDE-file for DIO Base with Tico                           #
'##############################################################################
'#                                                                            #
'#  08.08.2007  TB   creation                                                 #
'#  13.10.2009  ur   ECAT defines                                             #
'#  23.03.2010  ur   DIO-Base-Names changed                                   #
'#  10.12.2012  ur   ECAT defines added & changed                             #
'#  16.04.2016  mf   Calc_Processdelay
'#                                                                            #
'# Attention:                                                                 #
'#   - Do not copy the first source line (containing                          #
'#     "'ADBASIC-SYSTEM-FILE...") to user files !                             #
'#     Otherwise the Syntax color may be not correct.                         #
'#   - This includefile requires ADbasic 4.20.04 or higher!                   #
'#   - Some functions use critical sections; drop-in replacements without     #
'#     critical sections are available with suffix_No_Crit.                   #
'##############################################################################
'<ENDREGION>

'<REGION> Declaration

' This file contains defines that are used both on the DSP and on the TiCo. 

' DATAs on the TiCo used for communication with DSP. Since we use double
' buffering, we have 2 datas for input values and 2 datas for output
' values
' Note: The defines are intentionally not of the form #define
' ECAT_Inputs_2 Data_2, because the defines should be used for
' accessing the Data directly (on the TiCo) and using the P2-Bus (on the
' DSP)
' To use these defines on the TiCo side, use Data[ECAT_Inputs_1].
' To access these datas from the DSP, use P2_GetData_Long(..., 
' ECAT_Inputs_1).
#define ECAT_DSP_To_TiCo_1                    1
#define ECAT_DSP_To_TiCo_2                    2
#define ECAT_TiCo_To_DSP_1                    3
#define ECAT_TiCo_To_DSP_2                    4
#define ECAT_DSP_To_TiCo_11                   5
#define ECAT_DSP_To_TiCo_21                   6
#define ECAT_TiCo_To_DSP_11                   7
#define ECAT_TiCo_To_DSP_21                   8

' PARs used for interesting values and communication
' Note: The note about the Datas above does also apply here. Use
' Par[ECAT_Init_Result] on the TiCo, and P2_Get_Par(..., 
' ECAT_Init_Result) on the DSP.

''' Par no.: result of initialization function
#define ECAT_Init_Result                  1
''' Par no.: version of driver software in TiCo
#define ECAT_Driver_Version               2
''' Par no.: current state of the ECAT state machine
#define ECAT_Current_ECAT_State       3
''' Par no.: result of last read operation
#define ECAT_Last_Read_Result             4
''' Par no.: result of last write input operation
#define ECAT_Last_Write_Result            5
''' Par no.: a magic flag, value must be ECAT_Magic_Flag_Value
#define ECAT_Magic_Flag_Par               6
''' Par no.: current value of the distributed clock, low
#define ECAT_Current_DC_Time_Low          7
''' Par no.: current value of the distributed clock, high
#define ECAT_Current_DC_Time_High         8
''' Par no.: ECAT data type
'''+ State: 0-> Long values
'''+        1-> Float values
'''+        2-> Long and Float values
#define ECAT_Config_Value                 9
' parameters to control double buffering of input and output datas
' the buffer the DSP is currently allowed to write to
' 1: DSP can write to buffer 1
' 2: DSP can write to buffer 2
' 0: DSP finished writing
' TiCo writes 1 or 2 to this variable, the DSP polls it, writes to the data
' and resets the value to 0
#define ECAT_Input_Buffer_DSP            11
#define ECAT_Input_Buffer_TiCo           12
#define ECAT_Input_Buffer_DSP_1          13
#define ECAT_Input_Buffer_TiCo_1         14
' basically the same, just for writing the reading values
#define ECAT_Output_Buffer_DSP           21
#define ECAT_Output_Buffer_TiCo          22
#define ECAT_Output_Buffer_DSP_1         23
#define ECAT_Output_Buffer_TiCo_1        24

''' on ECAT cards, the ECAT_Magic_Flag_Par always holds this value
#define ECAT_Magic_Flag_Value         0AD0Eca1h

''' maximum size of input datas
#define ECAT_RXPDO_COUNT   32
''' maximum size of output datas
#define ECAT_TXPDO_COUNT   32

#define ECAT_STATE_INIT            0001h
#define ECAT_STATE_PREOP           0002h
#define ECAT_STATE_BOOT            0003h
#define ECAT_STATE_SAFEOP          0004h
#define ECAT_STATE_OP              0008h
'<ENDREGION>

'<REGION> Helper functions

'***************************************************************
''' Converts a process frequency [Hz] into processor ticks (processdelay or cycle time).
''' frequency:  frequency[Hz]
'''*************************************************************
Function Calc_Processdelay(frequency) as long
  
#if prozessor=tico1
  calc_processdelay=50000000/(frequency)
#else
  calc_processdelay=100000000/(frequency)
#endif
  
endfunction
'---------------------------------------------------------------
' If a low-priority process accesses a Pro-module it must not be
' interrupted by any other process !!!
'---------------------------------------------------------------

Sub Sub_Begin_Critical_Section()                            '<HIDE>
#If PRIORITAET = LOW
  Begin_Critical_Section
#EndIf
EndSub

Sub Sub_End_Critical_Section()                              '<HIDE>
#If PRIORITAET = LOW
  End_Critical_Section
#EndIf
EndSub

'<ENDREGION>

'<REGION> Get/Set direction of channels

'******************************************************************************
' Sets channels as input or output (channel 0 - 31). 
' pattern: each bit corresponds to one channel 
'     0 = input
'     1 = output
'******************************************************************************
Sub DigProg_Bits(pattern)
  out(4101H, pattern)
EndSub

'******************************************************************************
' Sets channels as input or output (channel 32 - 58). 
' pattern: each bit corresponds to one channel 
'     0 = input
'     1 = output
' Note: bits 28 - 32 are ignored
'******************************************************************************
Sub DigProg_Bits_High(pattern)
  out(4102H, pattern)
EndSub

'******************************************************************************
' Gets information about channels directions (channel 0 - 31). 
'     0 = input
'     1 = output
'******************************************************************************
Function Get_DigProg_Bits() As Long
  Get_DigProg_Bits = in(4101H)  
EndFunction

'******************************************************************************
' Gets information about channels directions (channel 32 - 58). 
'     0 = input
'     1 = output
' Note: bits 28 - 32 of return value are always 0
'******************************************************************************
Function Get_DigProg_Bits_High() As Long
  Get_DigProg_Bits_High = in(4102H)
EndFunction

'<ENDREGION>

'<REGION> Input functions

'******************************************************************************
' Reads digital inputs (channel 0 - 31) and write to input latch.
' Note: output channels are read as 0
' Note: updates input latch register
'******************************************************************************
Function Digin_Long() AS Long
  Digin_Long = in(100H)
EndFunction

'******************************************************************************
' Reads digital inputs (channel 32 - 58).
' Note: output channels are read as 0
' Note: updates input latch register
' Note: bits 28 - 32 are always 0
'******************************************************************************
Function Digin_Long_High() AS Long
  Digin_Long_High = in(101H)
EndFunction

'<ENDREGION>

'<REGION> Output functions

'******************************************************************************
' Sets digital outputs (channel 0 - 31).
' value: each bit corresponds to a channel
' Note: updates output latch register
'******************************************************************************
Sub Digout_Long(value)
  out(0, value)
EndSub

'******************************************************************************
' Sets digital outputs (channel 32 - 58). 
' value: each bit corresponds to a channel 
' Note: updates output latch register
' Note: bits 28 - 32 of value are ignored
'******************************************************************************
Sub Digout_Long_High(value)
  out(1, value)
EndSub

'******************************************************************************
' Sets and clears selected digital outputs (channel 0 - 31).
' set: Bitstring with outputs to set
' clear: Bitstring with outputs to clear
' Note: does NOT update output latch register
'******************************************************************************
Sub Digout_Bits(set, clear)
  Sub_Begin_Critical_Section()
  Digout_Bits_No_Crit(set, clear)
  Sub_End_Critical_Section()
EndSub

Sub Digout_Bits_No_Crit(set, clear)                         '<HIDE>
  out(32101000h, clear)
  out(32100000h, set) 
EndSub

'******************************************************************************
' Sets and clears selected digital outputs (channel 32 - 58).
' set: Bitstring with outputs to set
' clear: Bitstring with outputs to clear
' Note: does NOT update output latch register
' Note: bits 28 - 32 of set and clear are ignored
'******************************************************************************
Sub Digout_Bits_High(set, clear)
  Sub_Begin_Critical_Section()
  Digout_Bits_High_No_Crit(set, clear)
  Sub_End_Critical_Section()
EndSub

Sub Digout_Bits_High_No_Crit(set, clear)                    '<HIDE>
  out(32101001h, clear)
  out(32100001h, set)
EndSub

'******************************************************************************
' Sets or clears one channel of a digital output.
' channel: channel (0 - 31)
' value: 0 = clear channel
'        1 = set channel
' Note: does NOT update output latch register
'******************************************************************************
Sub Digout(channel, value)  
  out((0021h - value), (shift_left(1, (channel))))
EndSub

'******************************************************************************
' Sets or clears one channel of a digital output.
' channel: channel (32 - 58)
' value: 0 = clear channel
'        1 = set channel
' Note: does NOT update output latch register
'******************************************************************************
Sub Digout_High(channel, value)  
  out((0023h - value), (shift_left(1, (channel))))
EndSub

'******************************************************************************
' Sets the bits matched by mask to value (channels 0 - 31)
' mask: bitmask of channels to update
' value: bitmask of new outputs
' Note: updates output latch register
'******************************************************************************
Sub Digout_Change_Bits(mask, value)
  Sub_Begin_Critical_Section()
  Digout_Change_Bits_No_Crit(mask, value)
  Sub_End_Critical_Section()
EndSub

Sub Digout_Change_Bits_No_Crit(mask, value)                 '<HIDE>
  Dim tmp as Long
  ' get current outputs
  tmp = Get_Digout_Long()
  ' put everything together
  tmp = (tmp AND NOT (mask)) OR (mask AND value)
  ' set new outputs
  Digout_Long(tmp)
EndSub

'******************************************************************************
' Sets the bits matched by mask to value (channels 32 - 58).
' mask: bitmask of channels to update
' value: bitmask of new outputs
' Note: updates output latch register
'******************************************************************************
Sub Digout_Change_Bits_High(mask, value)
  Sub_Begin_Critical_Section()
  Digout_Change_Bits_High_No_Crit(mask, value)
  Sub_End_Critical_Section()
EndSub

Sub Digout_Change_Bits_High_No_Crit(mask, value)            '<HIDE>
  Dim tmp as Long
  ' get current outputs
  tmp = Get_Digout_Long_High()
  ' put everything together
  tmp = (tmp AND NOT (mask)) OR (mask AND value)
  ' set new outputs
  Digout_Long_High(tmp)
EndSub

'<ENDREGION>

'<REGION> Access latch registers

'******************************************************************************
' Reads back the current output value of the digital outputs (channel 32 - 58).
'******************************************************************************
Function Get_Digout_Long() AS Long
  Get_Digout_Long = in(0)
EndFunction

'******************************************************************************
' Reads back the current output value of the digital outputs (channel 32 - 58).
'******************************************************************************
Function Get_Digout_Long_High() AS Long
  Get_Digout_Long_High = in(1)  
EndFunction
 
'******************************************************************************
' Reads the input latch register (channel 0 - 31).
'******************************************************************************
Function Dig_Read_Latch() AS Long
  Dig_Read_Latch = in(110h)
EndFunction

'******************************************************************************
' Reads the input latch register (channel 32 - 58).
'******************************************************************************
Function Dig_Read_Latch_High() AS Long
  Dig_Read_Latch_High = in(111h)
EndFunction

'******************************************************************************
' Reads the output latch register (channel 0 - 31).
'******************************************************************************
Function Dig_Read_Output_Latch() AS Long
  Dig_Read_Output_Latch = in(10h)
EndFunction

'******************************************************************************
' Reads the output latch register (channel 32 - 58).
'******************************************************************************
Function Dig_Read_Output_Latch_High() AS Long
  Dig_Read_Output_Latch_High = in(11h)
EndFunction

'******************************************************************************
' Sets output latch register (channel 32 - 58).
' value: each bit corresponds to a channel
'******************************************************************************
Sub Dig_Write_Latch(pattern)
  out(10H, pattern)
EndSub

'******************************************************************************
' Sets output latch register (channel 32 - 58).
' value: each bit corresponds to a channel
'******************************************************************************
Sub Dig_Write_Latch_High(pattern)
  out(11H, pattern)
EndSub

'<ENDREGION>

