#include <stdlib.h>
#include <stdio.h>

#include "adwin-macros.h"
#include "libadwin-macros.h"
#include "data_convert.h"

void *Get_Data_prepare_convert(adwin_system_data_type_t asdt, void *src,
                               adwin_data_type_t adt, unsigned long count)
{
    void *conv;

    switch (adt) {
        case ADWIN_BYTE:
            switch (asdt) {
                case ADWIN_SYSTEM_BYTE:
                    return src;
                case ADWIN_SYSTEM_SHORT:
                    conv = malloc(sizeof(int16_t) * count);
                    return conv;
                case ADWIN_SYSTEM_INTEGER:
                case ADWIN_SYSTEM_LONG:
                case ADWIN_SYSTEM_FLOAT:
                    conv = malloc(sizeof(int32_t) * count);
                    return conv;
                case ADWIN_SYSTEM_DOUBLE:
                case ADWIN_SYSTEM_INT64:
                    conv = malloc(sizeof(double) * count);
                    return conv;
            }
        case ADWIN_SHORT:
            switch (asdt) {
                case ADWIN_SYSTEM_BYTE:
                    conv = malloc(sizeof(int8_t) * count);
                    return conv;
                case ADWIN_SYSTEM_SHORT:
                    return src;
                case ADWIN_SYSTEM_INTEGER:
                case ADWIN_SYSTEM_LONG:
                case ADWIN_SYSTEM_FLOAT:
                    conv = malloc(sizeof(int32_t) * count);
                    return conv;
                case ADWIN_SYSTEM_DOUBLE:
                case ADWIN_SYSTEM_INT64:
                    conv = malloc(sizeof(double) * count);
                    return conv;
            }
        case ADWIN_LONG:
        case ADWIN_INTEGER:
        case ADWIN_FLOAT:
            switch (asdt) {
                case ADWIN_SYSTEM_BYTE:
                    conv = malloc(sizeof(int8_t) * count);
                    return conv;
                case ADWIN_SYSTEM_SHORT:
                    conv = malloc(sizeof(int32_t) * count);
                    return conv;
                case ADWIN_SYSTEM_INTEGER:
                case ADWIN_SYSTEM_LONG:
                case ADWIN_SYSTEM_FLOAT:
                    return src;
                case ADWIN_SYSTEM_DOUBLE:
                case ADWIN_SYSTEM_INT64:
                    conv = malloc(sizeof(double) * count);
                    return conv;
            }
        case ADWIN_DOUBLE:
        case ADWIN_INT64:
            switch (asdt) {
                case ADWIN_SYSTEM_BYTE:
                    conv = malloc(sizeof(int8_t) * count);
                    return conv;
                case ADWIN_SYSTEM_SHORT:
                    conv = malloc(sizeof(int16_t) * count);
                    return conv;
                case ADWIN_SYSTEM_INTEGER:
                case ADWIN_SYSTEM_LONG:
                case ADWIN_SYSTEM_FLOAT:
                    conv = malloc(sizeof(int32_t) * count);
                    return conv;
                case ADWIN_SYSTEM_DOUBLE:
                case ADWIN_SYSTEM_INT64:
                    return src;
            }
        default:
            PRINTDEBUG(DBG_GENERIC_DRV, ("unknown adwin_data_type_t = %d\n", adt));
            return src;
    }
}

/**
 * Copy count elements of type asdt located at src to type adt located
 * at dest. If no conversion is necessary, this does nothing (not even
 * copying from source do dest, because in that case source and dest
 * should be the same!). Returns 0 if nothing was done, 1 otherwise.
 */
int Get_Data_do_convert(adwin_system_data_type_t asdt, void *src,
                        adwin_data_type_t adt, void *dest, unsigned long count)
{
    switch (adt) {
        case ADWIN_BYTE:
            switch (asdt) {
                case ADWIN_SYSTEM_BYTE:
                    return 0;
                case ADWIN_SYSTEM_SHORT:
                    CAST(int16_t, src, int8_t, dest, count);
                    return 1;
                case ADWIN_SYSTEM_INTEGER:
                case ADWIN_SYSTEM_LONG:
                    CAST(int32_t, src, int8_t, dest, count);
                    return 1;
                case ADWIN_SYSTEM_FLOAT:
                    CAST(float, src, int8_t, dest, count);
                    return 1;
                case ADWIN_SYSTEM_DOUBLE:
                    CAST(double, src, int8_t, dest, count);
                    return 1;
                case ADWIN_SYSTEM_INT64:
                    CAST(int64_t, src, int8_t, dest, count);
                    return 1;
            }
        case ADWIN_SHORT:
            switch (asdt) {
                case ADWIN_SYSTEM_BYTE:
                    CAST(int8_t, src, int16_t, dest, count);
                    return 1;
                case ADWIN_SYSTEM_SHORT:
                    return 0;
                case ADWIN_SYSTEM_INTEGER:
                case ADWIN_SYSTEM_LONG:
                    CAST(int32_t, src, int16_t, dest, count);
                    return 1;
                case ADWIN_SYSTEM_FLOAT:
                    CAST(float, src, int16_t, dest, count);
                    return 1;
                case ADWIN_SYSTEM_DOUBLE:
                    CAST(double, src, int16_t, dest, count);
                    return 1;
                case ADWIN_SYSTEM_INT64:
                    CAST(int64_t, src, int16_t, dest, count);
                    return 1;
            }
        case ADWIN_LONG:
            switch (asdt) {
                case ADWIN_SYSTEM_BYTE:
                    CAST(int8_t, src, int32_t, dest, count);
                    return 1;
                case ADWIN_SYSTEM_SHORT:
                    CAST(int16_t, src, int32_t, dest, count);
                    return 1;
                case ADWIN_SYSTEM_INTEGER:
                case ADWIN_SYSTEM_LONG:
                    return 0;
                case ADWIN_SYSTEM_FLOAT:
                    CAST(float, src, int32_t, dest, count);
                    return 1;
                case ADWIN_SYSTEM_DOUBLE:
                    CAST(double, src, int32_t, dest, count);
                    return 1;
                case ADWIN_SYSTEM_INT64:
                    CAST(int64_t, src, int32_t, dest, count);
                    return 1;
            }
        case ADWIN_FLOAT:
            switch (asdt) {
                case ADWIN_SYSTEM_BYTE:
                    CAST(int8_t, src, float, dest, count);
                    return 1;
                case ADWIN_SYSTEM_SHORT:
                    CAST(int16_t, src, float, dest, count);
                    return 1;
                case ADWIN_SYSTEM_INTEGER:
                case ADWIN_SYSTEM_LONG:
                    CAST(int32_t, src, float, dest, count);
                    return 1;
                case ADWIN_SYSTEM_FLOAT:
                    return 0;
                case ADWIN_SYSTEM_DOUBLE:
                    CAST(double, src, float, dest, count);
                    return 1;
                case ADWIN_SYSTEM_INT64:
                    CAST(int64_t, src, float, dest, count);
                    return 1;
            }
        case ADWIN_DOUBLE:
            switch (asdt) {
                case ADWIN_SYSTEM_BYTE:
                    CAST(int8_t, src, double, dest, count);
                    return 1;
                case ADWIN_SYSTEM_SHORT:
                    CAST(int16_t, src, double, dest, count);
                    return 1;
                case ADWIN_SYSTEM_INTEGER:
                case ADWIN_SYSTEM_LONG:
                    CAST(int32_t, src, double, dest, count);
                    return 1;
                case ADWIN_SYSTEM_FLOAT:
                    CAST(float, src, double, dest, count);
                    return 1;
                case ADWIN_SYSTEM_DOUBLE:
                    return 0;
                case ADWIN_SYSTEM_INT64:
                    CAST(int64_t, src, double, dest, count);
                    return 1;
            }
        case ADWIN_INT64:
            switch (asdt) {
                case ADWIN_SYSTEM_BYTE:
                    CAST(int8_t, src, int64_t, dest, count);
                    return 1;
                case ADWIN_SYSTEM_SHORT:
                    CAST(int16_t, src, int64_t, dest, count);
                    return 1;
                case ADWIN_SYSTEM_INTEGER:
                case ADWIN_SYSTEM_LONG:
                    CAST(int32_t, src, int64_t, dest, count);
                    return 1;
                case ADWIN_SYSTEM_FLOAT:
                    CAST(float, src, int64_t, dest, count);
                    return 1;
                case ADWIN_SYSTEM_DOUBLE:
                    CAST(double, src, int64_t, dest, count);
                    return 1;
                case ADWIN_SYSTEM_INT64:
                    return 0;
            }
        default:
            PRINTDEBUG(DBG_GENERIC_DRV, ("unknown adwin_data_type_t = %d\n", adt));
            return 0;
    }
}

/**
 * Copy count elements of type adt located at src to type asdt. If no
 * conversion is necessary, returns src, otherwise returns pointer to
 * malloc'ed memory that must be freed afterwards.
 *
 * @note: ideally, src would be const void*. But in some cases we
 * return src, so the whole function must have return type const
 * void*; however, since in other cases we return a pointer to
 * malloced memory, this function cannot be const :-(
 *
 * A possible solution would be two different functions for
 * Set_Data_convert - one that checks if src can be used directly, and
 * one that allocates memory if src cannot be used. However, for
 * performance reasons, we won't do it. Maybe such a function could be
 * implemented as a macro comparing sizeof(src) and sizeof(asdt)...
 */
void *Set_Data_convert(adwin_data_type_t adt, void *src,
                       adwin_system_data_type_t asdt, unsigned long count)
{
    void *conv;

    // todo: make sure that sizeof(float) = sizeof(int32_t) = 4 !

    switch (adt) {
        case ADWIN_BYTE:
            switch (asdt) {
                case ADWIN_SYSTEM_BYTE:
                    return src;
                case ADWIN_SYSTEM_SHORT:
                    conv = malloc(sizeof(int16_t) * count);
                    CAST(int8_t, src, int16_t, conv, count);
                    return conv;
                case ADWIN_SYSTEM_INTEGER:
                case ADWIN_SYSTEM_LONG:
                    conv = malloc(sizeof(int32_t) * count);
                    CAST(int8_t, src, int32_t, conv, count);
                    return conv;
                case ADWIN_SYSTEM_FLOAT:
                    conv = malloc(sizeof(float) * count);
                    CAST(int8_t, src, float, conv, count);
                    return conv;
                case ADWIN_SYSTEM_DOUBLE:
                    conv = malloc(sizeof(double) * count);
                    CAST(int8_t, src, double, conv, count);
                    return conv;
                case ADWIN_SYSTEM_INT64:
                    conv = malloc(sizeof(int64_t) * count);
                    CAST(int8_t, src, int64_t, conv, count);
                    return conv;
            }
        case ADWIN_SHORT:
            switch (asdt) {
                case ADWIN_SYSTEM_BYTE:
                    conv = malloc(sizeof(int8_t) * count);
                    CAST(int16_t, src, int8_t, conv, count);
                    return conv;
                case ADWIN_SYSTEM_SHORT:
                    return src;
                case ADWIN_SYSTEM_INTEGER:
                case ADWIN_SYSTEM_LONG:
                    conv = malloc(sizeof(int32_t) * count);
                    CAST(int16_t, src, int32_t, conv, count);
                    return conv;
                case ADWIN_SYSTEM_FLOAT:
                    conv = malloc(sizeof(float) * count);
                    CAST(int16_t, src, float, conv, count);
                    return conv;
                case ADWIN_SYSTEM_DOUBLE:
                    conv = malloc(sizeof(double) * count);
                    CAST(int16_t, src, double, conv, count);
                    return conv;
                case ADWIN_SYSTEM_INT64:
                    conv = malloc(sizeof(int64_t) * count);
                    CAST(int16_t, src, int64_t, conv, count);
                    return conv;
            }
        case ADWIN_LONG:
            switch (asdt) {
                case ADWIN_SYSTEM_BYTE:
                    conv = malloc(sizeof(int8_t) * count);
                    CAST(int32_t, src, int8_t, conv, count);
                    return conv;
                case ADWIN_SYSTEM_SHORT:
                    conv = malloc(sizeof(int16_t) * count);
                    CAST(int32_t, src, int16_t, conv, count);
                    return conv;
                case ADWIN_SYSTEM_INTEGER:
                case ADWIN_SYSTEM_LONG:
                    return src;
                case ADWIN_SYSTEM_FLOAT:
                    conv = malloc(sizeof(float) * count);
                    CAST(int32_t, src, float, conv, count);
                    return conv;
                case ADWIN_SYSTEM_DOUBLE:
                    conv = malloc(sizeof(double) * count);
                    CAST(int32_t, src, double, conv, count);
                    return conv;
                case ADWIN_SYSTEM_INT64:
                    conv = malloc(sizeof(int64_t) * count);
                    CAST(int32_t, src, int64_t, conv, count);
                    return conv;
            }
        case ADWIN_FLOAT:
            switch (asdt) {
                case ADWIN_SYSTEM_BYTE:
                    conv = malloc(sizeof(int8_t) * count);
                    CAST(float, src, int8_t, conv, count);
                    return conv;
                case ADWIN_SYSTEM_SHORT:
                    conv = malloc(sizeof(int16_t) * count);
                    CAST(float, src, int16_t, conv, count);
                    return conv;
                case ADWIN_SYSTEM_INTEGER:
                case ADWIN_SYSTEM_LONG:
                    conv = malloc(sizeof(int32_t) * count);
                    CAST(float, src, int32_t, conv, count);
                    return conv;
                case ADWIN_SYSTEM_FLOAT:
                    return src;
                case ADWIN_SYSTEM_DOUBLE:
                    conv = malloc(sizeof(double) * count);
                    CAST(float, src, double, conv, count);
                    return conv;
                case ADWIN_SYSTEM_INT64:
                    conv = malloc(sizeof(int64_t) * count);
                    CAST(float, src, int64_t, conv, count);
                    return conv;
            }
        case ADWIN_DOUBLE:
            switch (asdt) {
                case ADWIN_SYSTEM_BYTE:
                    conv = malloc(sizeof(int8_t) * count);
                    CAST(double, src, int8_t, conv, count);
                    return conv;
                case ADWIN_SYSTEM_SHORT:
                    conv = malloc(sizeof(int16_t) * count);
                    CAST(double, src, int16_t, conv, count);
                    return conv;
                case ADWIN_SYSTEM_INTEGER:
                case ADWIN_SYSTEM_LONG:
                    conv = malloc(sizeof(int32_t) * count);
                    CAST(double, src, int32_t, conv, count);
                    return conv;
                case ADWIN_SYSTEM_FLOAT:
                    conv = malloc(sizeof(float) * count);
                    CAST(double, src, float, conv, count);
                    return conv;
                case ADWIN_SYSTEM_DOUBLE:
                    return src;
                case ADWIN_SYSTEM_INT64:
                    conv = malloc(sizeof(int64_t) * count);
                    CAST(double, src, int64_t, conv, count);
                    return conv;
            }
        case ADWIN_INT64:
            switch (asdt) {
                case ADWIN_SYSTEM_BYTE:
                    conv = malloc(sizeof(int8_t) * count);
                    CAST(int64_t, src, int8_t, conv, count);
                    return conv;
                case ADWIN_SYSTEM_SHORT:
                    conv = malloc(sizeof(int16_t) * count);
                    CAST(int64_t, src, int16_t, conv, count);
                    return conv;
                case ADWIN_SYSTEM_INTEGER:
                case ADWIN_SYSTEM_LONG:
                    conv = malloc(sizeof(int32_t) * count);
                    CAST(int64_t, src, int32_t, conv, count);
                    return conv;
                case ADWIN_SYSTEM_FLOAT:
                    conv = malloc(sizeof(float) * count);
                    CAST(int64_t, src, float, conv, count);
                    return conv;
                case ADWIN_SYSTEM_DOUBLE:
                    conv = malloc(sizeof(double) * count);
                    CAST(int64_t, src, double, conv, count);
                    return conv;
                case ADWIN_SYSTEM_INT64:
                    return src;
            }
        default:
            PRINTDEBUG(DBG_GENERIC_DRV, ("unknown adwin_data_type_t = %d\n", adt));
            return src;
    }
}


void *memcpy_htolel(void *dest, const void *src, size_t n)
{
#ifdef ADWIN_BIG_ENDIAN
    int i;
    int32_t *lp_src, *lp_dest;
    lp_src  = (int32_t *)src;
    lp_dest = (int32_t *)dest;
    for (i = 0; i < n; i += sizeof(int32_t)) {
        *lp_dest = htolel(*lp_src);
        lp_dest++;
        lp_src++;
    }
    return dest; // compatible with memcpy
#else
    return memcpy(dest, src, n);
#endif
}

void *memcpy_letohl(void *dest, const void *src, size_t n)
{
#ifdef ADWIN_BIG_ENDIAN
    int i;
    int32_t *lp_dest;
    const int32_t *lp_src;
    lp_src = src;
    lp_dest = dest;
    for (i = 0; i < n; i += sizeof(int32_t)) {
        *lp_dest = letohl(*lp_src);
        lp_dest++;
        lp_src++;
    }
    return dest; // compatible with memcpy
#else
    return memcpy(dest, src, n);
#endif
}

